<?php
	// MyBB reCAPTCHA plugin code.
	// (C) 2010 CubicleSoft.  All Rights Reserved.
	//
	// This plugin is free to use with the MyBB forum software package.
	// Implements reCAPTCHA support.  Very useful if the GD extension is not available.
	// Dependencies:  reCAPTCHA library.
	// Creative Commons License:  http://creativecommons.org/licenses/by-nc-nd/3.0/us
	// Developed by Thomas Hruska, CubicleSoft Core.

	// Disallow direct access to this file for security reasons.
	if (!defined("IN_MYBB"))
	{
		die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
	}

	// Admin hooks.
	$plugins->add_hook("admin_config_settings_change", "reCAPTCHA_admin_config_settings");
	$plugins->add_hook("admin_config_settings_start", "reCAPTCHA_admin_config_settings");

	// Data processing hooks.
	$plugins->add_hook("datahandler_user_validate", "reCAPTCHA_datahandler_user_validate");
	$plugins->add_hook("datahandler_post_validate_post", "reCAPTCHA_datahandler_post_validate_postthread");
	$plugins->add_hook("datahandler_post_validate_thread", "reCAPTCHA_datahandler_post_validate_postthread");

	// Template modification hooks.
	$plugins->add_hook("member_do_login_start", "reCAPTCHA_member_do_login_start");
	$plugins->add_hook("member_register_start", "reCAPTCHA_member_register_start");
	$plugins->add_hook("newthread_start", "reCAPTCHA_main_start");
	$plugins->add_hook("newreply_start", "reCAPTCHA_main_start");
	$plugins->add_hook("showthread_start", "reCAPTCHA_main_start");

	function reCAPTCHA_info()
	{
		global $lang;

		$lang->load("reCAPTCHA_plugin_admin");

		return Array(
			"name" => $lang->reCAPTCHA_plugin_Name,
			"description" => $lang->reCAPTCHA_plugin_Desc,
			"website" => "http://mods.mybboard.net/view/recaptcha-ii",
			"author" => $lang->reCAPTCHA_plugin_Author,
			"authorsite" => "http://www.cubiclesoft.com/",
			"version" => "1.4",
			"guid" => "9ee8f9a34737a633c940348777c72fb0",
			"compatibility" => "16*"
		);
	}

	function reCAPTCHA_install()
	{
		global $mybb, $db;

		// Delete existing settings (if any).
		$query = $db->simple_select("settinggroups", "gid", "name = 'reCAPTCHA_plugin'");
		$gid = $db->fetch_field($query, "gid");
		if ($gid)  $db->delete_query("settings", "gid = '" . (int)$gid . "'");
		$db->delete_query("settinggroups", "name = 'reCAPTCHA_plugin'");

		// Create new settings group.
		$query = $db->simple_select("settinggroups", "COUNT(*) as rows");
		$numrows = $db->fetch_field($query, "rows");
		$insertarray = Array(
			"name" => "reCAPTCHA_plugin",
			"title" => "reCAPTCHA Configuration",
			"description" => "Sets up the basic reCAPTCHA options",
			"disporder" => $numrows + 1,
			"isdefault" => 0
		);
		$db->insert_query("settinggroups", $insertarray);
		$gid = $db->insert_id();

		// Create 'PublicKey' setting.
		$insertarray = Array(
			"name" => "reCAPTCHA_PublicKey",
			"title" => "Public Key",
			"description" => $db->escape_string("You get a public key when you <a href=\"https://www.google.com/recaptcha/admin/list\" target=\"_blank\">sign up for the reCAPTCHA service</a>.  reCAPTCHA will not work without a public key!"),
			"optionscode" => "text",
			"value" => "",
			"disporder" => 1,
			"gid" => (int)$gid
		);
		$db->insert_query("settings", $insertarray);

		// Create 'PrivateKey' setting.
		$insertarray = Array(
			"name" => "reCAPTCHA_PrivateKey",
			"title" => "Private Key",
			"description" => $db->escape_string("You get a private key when you <a href=\"https://www.google.com/recaptcha/admin/list\" target=\"_blank\">sign up for the reCAPTCHA service</a>.  reCAPTCHA will not work without a private key!"),
			"optionscode" => "text",
			"value" => "",
			"disporder" => 2,
			"gid" => (int)$gid
		);
		$db->insert_query("settings", $insertarray);

		// Create 'Style' setting.
		$insertarray = Array(
			"name" => "reCAPTCHA_Style",
			"title" => "Style",
			"description" => "Select the style to use.  The default style works well with most themes.",
			"optionscode" => "select
red=Red
white=White
blackglass=Black Glass
clean=Clean",
			"value" => "red",
			"disporder" => 3,
			"gid" => (int)$gid
		);
		$db->insert_query("settings", $insertarray);

		rebuild_settings();
	}

	function reCAPTCHA_is_installed()
	{
		global $db;

		$query = $db->simple_select("settings", "sid", "name = 'reCAPTCHA_Style'");
		$sid = $db->fetch_field($query, "sid");
		if ($sid)  return true;

		return false;
	}

	function reCAPTCHA_uninstall()
	{
		global $db;

		// Delete existing settings (if any).
		$query = $db->simple_select("settinggroups", "gid", "name = 'reCAPTCHA_plugin'");
		$gid = $db->fetch_field($query, "gid");
		if ($gid)  $db->delete_query("settings", "gid = '" . (int)$gid . "'");
		$db->delete_query("settinggroups", "name = 'reCAPTCHA_plugin'");

		rebuild_settings();
	}

	function reCAPTCHA_admin_config_settings()
	{
		global $lang;

		$lang->load("reCAPTCHA_plugin_admin");
	}

	function reCAPTCHA_datahandler_override(&$Handler)
	{
		global $mybb, $lang;

		// Only some validate_...() calls will contain reCAPTCHA data.
		if ($mybb->settings["captchaimage"] == 1 && trim($mybb->settings["reCAPTCHA_PublicKey"]) != "" && trim($mybb->settings["reCAPTCHA_PrivateKey"]) != "")
		{
			$lang->load("reCAPTCHA_plugin", true);
			if (!array_key_exists("recaptcha_challenge_field", $mybb->input) || !array_key_exists("recaptcha_response_field", $mybb->input))  $Handler->set_error("reCAPTCHA_plugin_Error_NoChallengeResponse");
			else
			{
				require_once("support/recaptchalib.php");

				$response = recaptcha_check_answer(trim($mybb->settings["reCAPTCHA_PrivateKey"]), get_ip(), $mybb->input["recaptcha_challenge_field"], $mybb->input["recaptcha_response_field"]);
				if (!($response->is_valid))  $Handler->set_error("reCAPTCHA_plugin_Error_Incorrect", Array($response->error));
			}

			// Temporarily disable the 'captchaimage' setting, which disables the original CAPTCHA.
			$mybb->settings["captchaimage"] = 0;

			// Create a fake setting for member.php when it falls through on failure of this or some other registration field.
			$mybb->settings["recaptchaimage"] = 1;
		}
	}

	// Called by admin/modules/user/users.php, member.php, modcp.php, and usercp.php via validate_user().
	// Only care about the member.php call.
	function reCAPTCHA_datahandler_user_validate(&$UserHandler)
	{
		global $current_page;

		if ($current_page == "member.php")  reCAPTCHA_datahandler_override($UserHandler);
	}

	// Called by editpost.php, newreply.php, newthread.php, and xmlhttp.php via validate_post()/validate_thread()
	function reCAPTCHA_datahandler_post_validate_postthread(&$PostHandler)
	{
		global $mybb;

		if (($current_page == "newreply.php" || $current_page == "newthread.php") && !$mybb->user["uid"])  reCAPTCHA_datahandler_override($PostHandler);
	}

	function reCAPTCHA_DumpDiv()
	{
		global $mybb;

		$content = '<div id="recaptcha_div"></div><noscript>
	<iframe src="http://www.google.com/recaptcha/api/noscript?k=' . urlencode(trim($mybb->settings["reCAPTCHA_PublicKey"])) . '" height="300" width="500" frameborder="0"></iframe><br>
	<textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
	<input type="hidden" name="recaptcha_response_field" value="manual_challenge">
</noscript>';

		return $content;
	}

	function reCAPTCHA_member_do_login_start()
	{
		global $db, $mybb, $templates, $theme, $lang, $inline_errors_recaptcha, $captcha, $recaptcha;

		if (!$mybb->user["uid"] && ($mybb->settings["captchaimage"] == 1 || $mybb->settings["recaptchaimage"] == 1) && trim($mybb->settings["reCAPTCHA_PublicKey"]) != "" && trim($mybb->settings["reCAPTCHA_PrivateKey"]) != "")
		{
			$inline_errors_recaptcha = "";
			$lang->load("reCAPTCHA_plugin");

			if ($mybb->input["quick_login"] == "1" && $mybb->input["quick_password"] && $mybb->input["quick_username"])  $mybb->input["username"] = $mybb->input["quick_username"];
			if (username_exists($mybb->input["username"]))
			{
				$query = $db->simple_select("users", "loginattempts", "LOWER(username)='" . $db->escape_string(my_strtolower($mybb->input['username'])) . "'", array("limit" => 1));
				$loginattempts = $db->fetch_field($query, "loginattempts");

				if ($mybb->settings['failedcaptchalogincount'] > 0 && ($loginattempts > $mybb->settings['failedcaptchalogincount'] || intval($mybb->cookies['loginattempts']) > $mybb->settings['failedcaptchalogincount']))
				{
					if (!array_key_exists("recaptcha_challenge_field", $mybb->input) || !array_key_exists("recaptcha_response_field", $mybb->input))
					{
						$errors = array($lang->userdata_reCAPTCHA_plugin_Error_NoChallengeResponse);
						$inline_errors_recaptcha = inline_error($errors);
					}
					else
					{
						require_once("support/recaptchalib.php");

						$response = recaptcha_check_answer(trim($mybb->settings["reCAPTCHA_PrivateKey"]), get_ip(), $mybb->input["recaptcha_challenge_field"], $mybb->input["recaptcha_response_field"]);
						if (!($response->is_valid))
						{
							$errors = array($lang->sprintf($lang->userdata_reCAPTCHA_plugin_Error_Incorrect, $response->error));
							$inline_errors_recaptcha = inline_error($errors);
						}
					}

					// Save a few lines of code.
					reCAPTCHA_main_start();

					// This rather evil code is needed because $captcha is set to a blank string and there is no way to get errors to display otherwise!
					if ($inline_errors_recaptcha != "")
					{
						$mybb->input["quick_password"] = "";
						$mybb->input["password"] = "";
					}
					$recaptcha = $captcha;
					$content = $templates->get("member_login", 0, 0);
					$content = str_replace('{$inline_errors}', '{$inline_errors}{$inline_errors_recaptcha}', $content);
					$content = str_replace('{$captcha}', '{$captcha}{$recaptcha}', $content);
					$templates->cache["member_login"] = $content;
				}
			}
		}
	}

	function reCAPTCHA_member_register_start()
	{
		global $mybb, $templates, $lang;

		if (($mybb->settings["captchaimage"] == 1 || $mybb->settings["recaptchaimage"] == 1) && trim($mybb->settings["reCAPTCHA_PublicKey"]) != "" && trim($mybb->settings["reCAPTCHA_PrivateKey"]) != "")
		{
			$lang->load("reCAPTCHA_plugin");
			$templates->get("member_register_regimage", 0, 0);
			$content = '<fieldset class="trow2">' . reCAPTCHA_DumpFix() . '
<legend><strong>{$lang->image_verification}</strong></legend>
<table cellspacing="0" cellpadding="{$theme[' . "'tablespace'" . ']}">
<tr>
<td><span class="smalltext">{$lang->reCAPTCHA_verification_note}</span><br /><br />' . reCAPTCHA_DumpDiv() . '</td>
</tr>
</table>
</fieldset>';
			$templates->cache["member_register_regimage"] = $content;
			eval("\$GLOBALS[\"regimage\"] = \"" . $templates->get("member_register_regimage") . "\";");

			// Temporarily disable the 'captchaimage' setting, which disables the original CAPTCHA.
			$mybb->settings["captchaimage"] = 0;
		}
	}

	function reCAPTCHA_main_start()
	{
		global $mybb, $templates, $theme, $lang;

		if (!$mybb->user["uid"] && ($mybb->settings["captchaimage"] == 1 || $mybb->settings["recaptchaimage"] == 1) && trim($mybb->settings["reCAPTCHA_PublicKey"]) != "" && trim($mybb->settings["reCAPTCHA_PrivateKey"]) != "")
		{
			$lang->load("reCAPTCHA_plugin");
			$templates->get("post_captcha", 0, 0);
			$content = '<tr id="captcha_trow">
<td class="trow1" valign="top"><strong>{$lang->image_verification}</strong></td>
<td class="trow1">' . reCAPTCHA_DumpFix() . '
<table style="width: 300px; padding: 4px;">
<tr>
<td><span class="smalltext">{$lang->reCAPTCHA_verification_note}</span><br /><br />' . reCAPTCHA_DumpDiv() . '</td>
</tr>
</table>
</td>
</tr>';
			$templates->cache["post_captcha"] = $content;
			eval("\$GLOBALS[\"captcha\"] = \"" . $templates->get("post_captcha") . "\";");

			// Temporarily disable the 'captchaimage' setting, which disables the original CAPTCHA.
			$mybb->settings["captchaimage"] = 0;
		}
	}

	function reCAPTCHA_DumpFix()
	{
		global $mybb, $lang;

		// Determine the language.
		switch ($lang->language)
		{
			case 'dutch':  $TempLang = "nl";  break;
			case 'french':  $TempLang = "fr";  break;
			case 'deutsch_sie':  $TempLang = "de";  break;
			case 'deutsch_du':  $TempLang = "de";  break;
			case 'portugues_brasil':  $TempLang = "pt";  break;
			case 'russian':  $TempLang = "ru";  break;
			case 'espanol':  $TempLang = "es";  break;
			case 'turkish':  $TempLang = "tr";  break;
			default:  $TempLang = "en";
		}

		// Custom translation override (if specified).  See:  http://code.google.com/apis/recaptcha/docs/customization.html
		$custom = array();
		if (isset($lang->reCAPTCHA_custom_instructions_visual))  $custom["instructions_visual"] = $lang->reCAPTCHA_custom_instructions_visual;
		if (isset($lang->reCAPTCHA_custom_instructions_audio))   $custom["instructions_audio"] = $lang->reCAPTCHA_custom_instructions_audio;
		if (isset($lang->reCAPTCHA_custom_play_again))           $custom["play_again"] = $lang->reCAPTCHA_custom_play_again;
		if (isset($lang->reCAPTCHA_custom_cant_hear_this))       $custom["cant_hear_this"] = $lang->reCAPTCHA_custom_cant_hear_this;
		if (isset($lang->reCAPTCHA_custom_visual_challenge))     $custom["visual_challenge"] = $lang->reCAPTCHA_custom_visual_challenge;
		if (isset($lang->reCAPTCHA_custom_audio_challenge))      $custom["audio_challenge"] = $lang->reCAPTCHA_custom_audio_challenge;
		if (isset($lang->reCAPTCHA_custom_refresh_btn))          $custom["refresh_btn"] = $lang->reCAPTCHA_custom_refresh_btn;
		if (isset($lang->reCAPTCHA_custom_help_btn))             $custom["help_btn"] = $lang->reCAPTCHA_custom_help_btn;
		if (isset($lang->reCAPTCHA_custom_incorrect_try_again))  $custom["incorrect_try_again"] = $lang->reCAPTCHA_custom_incorrect_try_again;
		if (isset($lang->reCAPTCHA_custom_ISO_code))             $TempLang = $lang->reCAPTCHA_custom_ISO_code;
		if (!count($custom))  $custom = "";
		else
		{
			$temp = "";
			foreach ($custom as $key => $val)  $temp .= ($temp != "" ? "," : "") . "\n\t\t\t\t" . $key . " : \"" . addslashes($val) . "\"";
			$custom = ",\n\t\t\tcustom_translations : {" . $temp . "\n\t\t\t}";
		}

		$content = '<script type="text/javascript" src="http://www.google.com/recaptcha/api/js/recaptcha_ajax.js"></script>
<script type="text/javascript">
<!--
	// Fix for Prototype vs. weird reCAPTCHA JS library in Internet Explorer "Operation Aborted" DOM modification error.
	// Caused by a combination of the reCAPTCHA folks being dumb (refusal to fix the problem) and IE being retarded.
	Event.observe(window, "load", function()  { reCAPTCHA_PrototypeFix() });

	function reCAPTCHA_PrototypeFix()
	{
		Recaptcha.create("' . addslashes(trim($mybb->settings["reCAPTCHA_PublicKey"])) . '", "recaptcha_div", {
			theme : "' . $mybb->settings["reCAPTCHA_Style"] . '",
			lang : "' . addslashes($TempLang) . '",
			tabindex : 0' . $custom . '
		});
	}
// -->
</script>';

		return $content;
	}
?>